<?php
/*
Plugin Name: Paid Memberships Pro Pagadito Payment Gateway
Plugin URI: http://dev.pagadito.com
Description: Pasarela de Pago Pagadito for Paid Memberships Pro
Version: 1.0
Author: Pagadito Developers
Author URI: https://www.pagadito.com
License: GPL2
*/

/**
 * Pagadito.
 *
 * Pagadito Payment Gateway para Paid Memberships Pro
 *
 * Este programa es Software Libre: Usted puede redistribuirlo y/o modificarlo
 * bajo los terminos de la Licencia Publica General Reducida de GNU (GNU Lesser
 * Public Licence), tal como se encuentra publicada por la Free Software
 * Foundation, ya sea por su version 3 o cualquier otra version superior.
 *
 * Este programa es distribuido en el espiritu de que sea util, pero SIN NINGUNA
 * GARANTIA; sin tampoco garantia implicita de MERCANTIBILIDAD o ADAPTABILIDAD
 * PARA UN USO PARTICULAR. Vea la licencia GNU LGPL para mayores detalles.
 *
 * @category    Local
 * @package     Pagadito
 * @copyright   Copyright (c) 2018 - Pagadito, S. de R.L. (https://www.pagadito.com)
 * @license     http://www.gnu.org/licenses/lgpl.html
 * @author      Pagadito Development Team <developers@pagadito.com>
 */
require_once(dirname(__FILE__) . "/../paid-memberships-pro/classes/gateways/class.pmprogateway.php");

// Nota: el nombre de la clase en el api fue cambiada a PagaditoPMP para evitar
//       problemas de compatibilidad con otros plugins instalados que utilicen
//       la api de pagadito, cualquier actualizacion de api debe contemplar
//       este cambio o generara un error
require_once(dirname(__FILE__) . "/Pagadito.php");

//Wordpress init
add_action('init', array('PMProGateway_pagadito', 'init'));

// Hook para validar transacciones
add_action('wp_ajax_nopriv_pggetstatus', array('PMProGateway_pagadito', 'pagadito_get_status'));
add_action('wp_ajax_pggetstatus', array('PMProGateway_pagadito', 'pagadito_get_status'));

// hooks para recepción de webhooks
add_action('wp_ajax_nopriv_pgwebhooklistener', array('PMProGateway_pagadito', 'pagadito_webhook_listener'));
add_action('wp_ajax_pgwebhooklistener', array('PMProGateway_pagadito', 'pagadito_webhook_listener'));

/**
 * Clase principal de payment gateway
 */
class PMProGateway_pagadito extends PMProGateway {

    /**
     * Constructor de clase
     * @param mixed $gateway
     * @return mixed
     */
    function __construct($gateway = NULL) {

        $this->gateway = $gateway;
        return $this->gateway;
    }

    /**
     * Inicializador
     */
    static function init() {

        // Agregar pagadito como un payment gateway
        add_filter('pmpro_gateways', array('PMProGateway_pagadito', 'pmpro_gateways'));

        // Agrega pagadito pasarela de pago
        add_filter('pmpro_payment_options', array('PMProGateway_pagadito', 'pmpro_payment_options'));

        // Opciones administrativas
        add_filter('pmpro_payment_option_fields', array('PMProGateway_pagadito', 'pmpro_payment_option_fields'), 10, 2);

        //code to add at checkout
        $gateway = pmpro_getGateway();
        if ($gateway == "pagadito") {

            // Eliminar de pantalla de pago la direccion de facturacion
            add_filter('pmpro_include_billing_address_fields', '__return_false');

            // Quitar campos de pago en checkout
            add_filter('pmpro_include_payment_information_fields', '__return_false');

            // Modificar los campos necesarios en checkoput
            add_filter('pmpro_required_billing_fields', array('PMProGateway_pagadito', 'pmpro_required_billing_fields'));
            
            // Modifica los botones a mostrar en el proceso de pago
            add_filter('pmpro_checkout_default_submit_button', array('PMProGateway_pagadito', 'pmpro_checkout_default_submit_button'));

            // Ejecutar despues de procesar pago y antes de cambiar la membresia
            add_filter('pmpro_checkout_before_change_membership_level', array('PMProGateway_pagadito', 'pmpro_checkout_before_change_membership_level'), 10, 2);
        }
    }

    /**
     * Agrega pagadito como payment gateway
     * @param mixed $gateways
     * @return mixed
     */
    static function pmpro_gateways($gateways) {

        if (empty($gateways['pagadito']))
            $gateways['pagadito'] = __('Pagadito', 'paid-memberships-pro');

        return $gateways;
    }

    /**
     * Define las variables de configuracion del plugin
     * @return mixed
     */
    static function getGatewayOptions() {

        $options = array(
            'pg-uid',
            'pg-wsk',
            'currency',
            'use_ssl',
            'tax_state',
            'tax_rate'
        );
        return $options;
    }

    /**
     * Agrega y/o quita las opciones necesarias en la pantalla de pago
     * @param mixed $options
     * @return mixed
     */
    static function pmpro_payment_options($options) {

        $pagadito_options = PMProGateway_pagadito::getGatewayOptions();
        $options = array_merge($pagadito_options, $options);
        return $options;
    }

    /**
     * Agrega los parametros de configuracion especificos del plugin en el administrador
     * @param mixed $values
     * @param mixed $gateway
     */
    static function pmpro_payment_option_fields($values, $gateway) {
        global $pmpro_currencies;
            
        $pmpro_currencies['GTQ'] = array(
            'name' => __('Guatemalan Quetzal (Q)', 'paid-memberships-pro'),
            'symbol' => 'Q',
            'position' => 'left'
        );
        $pmpro_currencies['HNL'] = array(
            'name' => __('Honduran Lempira (L)', 'paid-memberships-pro'),
            'symbol' => 'L',
            'position' => 'left'
        );
        $pmpro_currencies['NIO'] = array(
            'name' => __('Nicaraguan Córdoba (C&#36;)', 'paid-memberships-pro'),
            'symbol' => 'C&#36;',
            'position' => 'left'
        );
        $pmpro_currencies['CRC'] = array(
            'name' => __('Costa Rican Colón (&#8353;)', 'paid-memberships-pro'),
            'symbol' => '&#8353;',
            'position' => 'left'
        );
        $pmpro_currencies['PAB'] = array(
            'name' => __('Panamanian balboa (B/.)', 'paid-memberships-pro'),
            'symbol' => 'B/.',
            'position' => 'left'
        );
        $pmpro_currencies['DOP'] = array(
            'name' => __('Dominican peso (&#36;)', 'paid-memberships-pro'),
            'symbol' => '&#36;',
            'position' => 'left'
        );
        ?>
        <tr class="pmpro_settings_divider gateway gateway_pagadito" <?php if ($gateway != "pagadito") { ?>style="display: none;"<?php } ?>>
            <td colspan="2">
                <?php _e('Configuraciones Pagadito', 'paid-memberships-pro'); ?>
            </td>
        </tr>
        <tr class="gateway gateway_pagadito" <?php if ($gateway != "pagadito") { ?>style="display: none;"<?php } ?>>
            <th scope="row" valign="top">
                <label for="pg-uid"><?php _e('UID', 'paid-memberships-pro'); ?>:</label>
            </th>
            <td>
                <input type="text" id="pg-uid" name="pg-uid" size="32" value="<?php echo esc_attr($values['pg-uid']) ?>" />
            </td>
        </tr>
        <tr class="gateway gateway_pagadito" <?php if ($gateway != "pagadito") { ?>style="display: none;"<?php } ?>>
            <th scope="row" valign="top">
                <label for="pg-wsk"><?php _e('WSK', 'paid-memberships-pro'); ?>:</label>
            </th>
            <td>
                <input type="text" id="pg-wsk" name="pg-wsk" size="32" value="<?php echo esc_attr($values['pg-wsk']) ?>" />
            </td>
        </tr>
        <tr class="gateway gateway_pagadito" <?php if ($gateway != "pagadito") { ?>style="display: none;"<?php } ?>>
            <th scope="row" valign="top">
                <label><?php _e('URL de retorno', 'paid-memberships-pro'); ?>:</label>
            </th>
            <td>
                <p><?php _e('Esta es su url de retorno para referencia. Usted debe setear esta url de retorno en su configuración técnica en su cuenta Pagadito.', 'paid-memberships-pro' );?> <pre><?php echo admin_url("admin-ajax.php") . "?action=pggetstatus&token={value}&order={ern_value}";?></pre></p>
            </td>
        </tr>
        <tr class="gateway gateway_pagadito" <?php if ($gateway != "pagadito") { ?>style="display: none;"<?php } ?>>
            <th scope="row" valign="top">
                <label><?php _e('URL de webhooks', 'paid-memberships-pro'); ?>:</label>
            </th>
            <td>
                <p><?php _e('Para una integración completa con Pagadito, asegurate de tener habilitado el envío de notificaciones vía webhook y usar la siguiente URL de webhook en la sección de webhooks en su cuenta Pagadito.', 'paid-memberships-pro' );?> <pre><?php echo admin_url("admin-ajax.php") . "?action=pgwebhooklistener";?></pre></p>
            </td>
        </tr>
        <script>
            document.addEventListener('DOMContentLoaded', function() {
                pmpro_changeGateway('pagadito');
             }, false);
        </script>
        <?php
    }

    /**
     * Elimina campos no necesarios de la pantalla de pago
     * @param mixed $fields
     * @return mixed
     */
    static function pmpro_required_billing_fields($fields) {

        unset($fields['bfirstname']);
        unset($fields['blastname']);
        unset($fields['baddress1']);
        unset($fields['bcity']);
        unset($fields['bstate']);
        unset($fields['bzipcode']);
        unset($fields['bphone']);
        unset($fields['bemail']);
        unset($fields['bcountry']);
        unset($fields['CardType']);
        unset($fields['AccountNumber']);
        unset($fields['ExpirationMonth']);
        unset($fields['ExpirationYear']);
        unset($fields['CVV']);
        return $fields;
    }
    
    static function pmpro_checkout_default_submit_button($show) {
        global $gateway, $pmpro_requirebilling;
        
        //show our submit buttons
        ?>
        <span id="pmpro_pagadito_checkout" <?php if($gateway != "pagadito" || !$pmpro_requirebilling) { ?>style="display: none;"<?php } ?>>
            <input type="hidden" name="submit-checkout" value="1" />
            <input type="image" value="<?php _e('Pagar con Pagadito', 'paid-memberships-pro' );?> &raquo;" src="<?php echo apply_filters("pmpro_pagadito_button_image", plugins_url() . "/pmprogateway_pagadito/buttons/pg-boton-pagar.jpg");?>" />
        </span>
        <span id="pmpro_pagadito_span" <?php if($gateway == "pagadito" && $pmpro_requirebilling) { ?>style="display: none;"<?php } ?>>
            <input type="hidden" name="submit-checkout" value="1" />
            <input type="submit" class="pmpro_btn pmpro_btn-submit-checkout" value="<?php _e('Submit and Confirm', 'paid-memberships-pro' ); ?> &raquo;" />
        </span>
        <?php

        //don't show the default
        return false;
    }

    /**
     * Asocia el usuario a la member order y redirecciona a la pantalla de pago de pagadito
     * @global string $discount_code_id
     * @global mixed $wpdb
     * @param integer $user_id
     * @param string $morder
     * @return bool
     */
    static function pmpro_checkout_before_change_membership_level($user_id, $morder) {
        global $discount_code_id, $wpdb;

        //if no order, no need to pay
        if (empty($morder))
            return;

        // Asociar usuario a la member order
        $morder->user_id = $user_id;
        $morder->saveOrder();

        //save discount code use
        if (!empty($discount_code_id))
            $wpdb->query("INSERT INTO $wpdb->pmpro_discount_codes_uses (code_id, user_id, order_id, timestamp) VALUES('" . $discount_code_id . "', '" . $user_id . "', '" . $morder->id . "', now())");

        do_action("pmpro_before_send_to_pagadito", $user_id, $morder);

        $morder->Gateway->sendToPagadito($morder);
    }

    /**
     * Inicia el proceso de la transaccion
     * @param mixed $order
     * @return bool
     */
    function process(&$order) {

        // if no order, no need to pay
        if (empty($order))
            return;

        // Si la transaccion no tiene un codigo generado previamente se le es asignado uno aleatorio
        if (empty($order->code))
            $order->code = $order->getRandomCode();

        // clean up a couple values
        $order->payment_type = "Pagadito";
        $order->CardType = "";
        $order->cardtype = "";

        //just save, the user will go to Pagadito to pay
        $order->status = "pending";
        $order->saveOrder();

        return true;
    }

    /**
     * Ejecuta la transaccion en pagadito
     * @param mixed $order
     * @return boolean
     */
    function sendToPagadito(&$order) {
        global $pmpro_currencies;

        // Se calcula el monto a pagar del pago inicial
        $initial_payment = $order->InitialPayment;
        $initial_payment_tax = $order->getTaxForPrice($initial_payment);
        $initial_payment = round((float) $initial_payment + (float) $initial_payment_tax, 2);

        // impuestos en el monto total a pagar
        $amount = $order->PaymentAmount;
        $amount_tax = $order->getTaxForPrice($amount);
        $amount = round((float) $amount + (float) $amount_tax, 2);

        // calcular fecha de inicio de la recurrencia
        $fecha_actual = new DateTime();

        // validar si la suscripción es recurring o un solo pago
        if (pmpro_isLevelRecurring($order->membership_level)) {

            // pago de suscripción con recurrencia
            // generara objeto de cobro inicial
            $obj_initial_charge = new stdClass();
            $obj_initial_charge->ern = $order->code;
            $obj_initial_charge->description = $order->membership_level->name . ' at ' . get_bloginfo("name");
            $obj_initial_charge->date = $fecha_actual->format('Y-m-d');
            $obj_initial_charge->quantity = 1;
            $obj_initial_charge->amount = $initial_payment;
            
            // obtener la frecuencia del periodo de pago
            $frequency = $order->BillingFrequency;

            // convertir el periodo de pago
            if ($order->BillingPeriod == "Day") {

                $period = "D";
            } elseif ($order->BillingPeriod == "Week") {

                $period = "W";
            } elseif ($order->BillingPeriod == "Month") {

                $period = "M";
            } elseif ($order->BillingPeriod == "Year") {

                $period = "Y";
            } else {

                $order->error = "Invalid billing period: " . $order->BillingPeriod;
                $order->shorterror = "Invalid billing period: " . $order->BillingPeriod;
                $order->status = "cancelled";
                $order->saveOrder();
                wp_redirect(pmpro_url("checkout", "?level=" . $order->membership_level->id));
                exit();
                return false;
            }

            // numero de recurrencias a cobrar
            if (!empty($order->TotalBillingCycles)) {

                $num_billing_cycles = intval($order->TotalBillingCycles);

                // si el billing_cycles es menor a 1 no es soportado
                if ($num_billing_cycles < 1) {

                    $order->error = "Invalid billing cycle limit: " . $order->TotalBillingCycles;
                    $order->shorterror = "Invalid billing cycle limit: " . $order->TotalBillingCycles;
                    $order->status = "cancelled";
                    $order->saveOrder();
                    wp_redirect(pmpro_url("checkout", "?level=" . $order->membership_level->id));
                    exit();
                    return false;
                }
            } else {

                $order->error = "Invalid billing cycle limit: " . $order->TotalBillingCycles . ".<br>Pagadito don't support memberships with indefinite billing cycles.";
                $order->shorterror = "Invalid billing cycle limit: " . $order->TotalBillingCycles;
                $order->status = "cancelled";
                $order->saveOrder();
                wp_redirect(pmpro_url("checkout", "?level=" . $order->membership_level->id));
                exit();
                return false;
            }
            
            // validar que no tenga un periodo de prueba
            if (!empty($order->TrialBillingPeriod)) {
                
                $order->error = "Pagadito don't support memberships with trial billing periods.";
                $order->shorterror = "Pagadito don't support memberships with trial billing periods.";
                $order->status = "cancelled";
                $order->saveOrder();
                wp_redirect(pmpro_url("checkout", "?level=" . $order->membership_level->id));
                exit();
            }
            
            // generar array con información de las recurrencias a programar
            $array_pending_charges = array();
            for ($charge = 1; $charge <= $num_billing_cycles; $charge++) {

                $obj_pending_charge = new stdClass();
                $obj_pending_charge->ern = $order->code . '-P' . $charge;
                $obj_pending_charge->description = $order->membership_level->name . ' at ' . get_bloginfo("name");
                $fecha_actual->add(new DateInterval('P' . $frequency . $period));
                $obj_pending_charge->date = $fecha_actual->format('Y-m-d');
                $obj_pending_charge->quantity = 1;
                $obj_pending_charge->amount = $amount;
                array_push($array_pending_charges, $obj_pending_charge);
                unset($obj_pending_charge);
            }
        } else {

            // generara objeto de cobro inicial
            $obj_initial_charge = new stdClass();
            $obj_initial_charge->ern = $order->code;
            $obj_initial_charge->description = $order->membership_level->name . ' at ' . get_bloginfo("name");
            $obj_initial_charge->date = $fecha_actual->format('Y-m-d');
            $obj_initial_charge->quantity = 1;
            $obj_initial_charge->amount = $initial_payment;
        }

        // pago de suscripción un solo pago
        try {

            // instancia de la api pagadito
            $pg = new PagaditoPMP(pmpro_getOption("pg-uid"), pmpro_getOption("pg-wsk"));

            // Estable el ambiente de conexion
            $environment = pmpro_getOption("gateway_environment");
            if ($environment === "sandbox") {

                $pg->mode_sandbox_on();
            }

            // Cambia al tipo de moneda deseado
            switch (pmpro_getOption("currency")) {
                
                case "USD":

                    $pg->change_currency_usd();
                    break;
                
                case "GTQ":

                    $pg->change_currency_gtq();
                    break;

                case "HNL":

                    $pg->change_currency_hnl();
                    break;

                case "NIO":

                    $pg->change_currency_nio();
                    break;

                case "CRC":

                    $pg->change_currency_crc();
                    break;

                case "PAB":

                    $pg->change_currency_pab();
                    break;

                case "DOP":

                    $pg->change_currency_dop();
                    break;

                default:

                    $order->error = "Pagadito don't support the currency " . $pmpro_currencies[pmpro_getOption("currency")]['name'] . ".";
                    $order->shorterror = "Pagadito don't support the currency " . $pmpro_currencies[pmpro_getOption("currency")]['name'] . ".";
                    $order->status = "cancelled";
                    $order->saveOrder();
                    wp_redirect(pmpro_url("checkout", "?level=" . $order->membership_level->id));
                    exit();
                    break;
            }

            // Conectando con pagadito
            if ($pg->connect()) {

                $redirect_url = false;

                // validar si la suscripción es recurring o un solo pago
                if (pmpro_isLevelRecurring($order->membership_level)) {

                    // pago inicial
                    $pg->add_detail($obj_initial_charge->quantity, $obj_initial_charge->description, $obj_initial_charge->amount);
                    $pg->add_pending_charge($obj_initial_charge->ern, $obj_initial_charge->description, $obj_initial_charge->date);

                    // recurrencias futuras
                    foreach ($array_pending_charges as $obj_pending_charge) {

                        $pg->add_detail($obj_pending_charge->quantity, $obj_pending_charge->description, $obj_pending_charge->amount);
                        $pg->add_pending_charge($obj_pending_charge->ern, $obj_pending_charge->description, $obj_pending_charge->date);
                    }
                    $redirect_url = $pg->authorization_recurring_payments('automatic_charges,initial_payment', false);
                } else {

                    $pg->add_detail($obj_initial_charge->quantity, $obj_initial_charge->description, $obj_initial_charge->amount);
                    $redirect_url = $pg->exec_trans($obj_initial_charge->ern, false);
                }

                // Ejecutando la transaccion en pagadito
                if ($redirect_url !== false) {

                    // Redireccionar a pagadito
                    wp_redirect($redirect_url);
                    exit();
                } else {

                    error_log("exec_trans or authorization_recurring_payments cannot execute");
                }
            } else {

                error_log("exec_trans or authorization_recurring_payments no connect");
            }
        } catch (Exception $e) {

            // Se detecto una excepcion ejecutando la transaccion
            error_log("exec_trans or authorization_recurring_payments exception = " . $e->getMessage());
        }

        // Mensaje de error generico
        $order->error = "Pagadtito: unknown error the transaction cannot be made, please try again.";
        $order->shorterror = "Pagadtito: unknown error the transaction cannot be made, please try again";
        $order->status = "cancelled";
        $order->saveOrder();
        wp_redirect(pmpro_url("checkout", "?level=" . $order->membership_level->id));
        exit();
        return false;
    }

    /**
     * Obtiene el estado de la transaccion y actualiza el member order
     */
    static function pagadito_get_status() {

        $token = $_GET['token'];
        $order = $_GET['order'];
        try {

            $morder = new MemberOrder($order); // Cargar orden retornada por pagadito
            $morder->getMembershipLevel(); // Cargar la membresia
            $morder->getUser(); // Cargar el usuario
            
            // instanciar API Pagadito
            $pg = new PagaditoPMP(pmpro_getOption("pg-uid"), pmpro_getOption("pg-wsk"));

            // Definiendo ambiente de conexion
            $environment = pmpro_getOption("gateway_environment");
            if ($environment === "sandbox") {

                $pg->mode_sandbox_on();
            }

            // Hay conexion con pagadito
            if ($pg->connect()) {

                // validar si la suscripción es recurring o un solo pago
                if (pmpro_isLevelRecurring($morder->membership_level)) {

                    // consultar token de pago recurrente
                    if ($pg->get_oauth_recurring_payments_token($token) === true) {

                        // obtener token de autorización
                        $suscription_transaction_id = $pg->get_rs_oauth_token();

                        // obtener cobros pendientes para validar que el primero fue completado
                        $array_cobros_registrados = $pg->get_rs_pending_charges();
                        $token_pending_initial_payment = $array_cobros_registrados[0]->token_pending;

                        // eliminar objeto pg
                        unset($pg);

                        // instanciar API Pagadito
                        $pg = new PagaditoPMP(pmpro_getOption("pg-uid"), pmpro_getOption("pg-wsk"));

                        // Definiendo ambiente de conexion
                        $environment = pmpro_getOption("gateway_environment");
                        if ($environment === "sandbox") {

                            $pg->mode_sandbox_on();
                        }

                        // realizar nuevo connect para consultar el estado del pago
                        $pg->connect();

                        // consultar el estado del pago inicial
                        if ($pg->get_pending_status($token_pending_initial_payment)) {

                            // obtener estado
                            switch ($pg->get_rs_status()) {

                                case "VERIFYING":

                                    // La transaccion esta pendiente de revision en pagadito
                                    $morder->status = "review";
                                    $morder->payment_transaction_id = $pg->get_rs_reference();
                                    $morder->subscription_transaction_id = $suscription_transaction_id;
                                    $morder->saveOrder();
                                    $redir_url = pmpro_url("confirmation", "?level=" . $morder->membership_level->id);
                                    break;

                                case "COMPLETED":

                                    // Transaccion completada con exito
                                    $morder->status = "success";
                                    $morder->payment_transaction_id = $pg->get_rs_reference();
                                    $morder->subscription_transaction_id = $suscription_transaction_id;
                                    $morder->saveOrder();
                                    $redir_url = pmpro_url("confirmation", "?level=" . $morder->membership_level->id);
                                    if (!self::pagadito_set_membership($morder)) {
                                        $redir_url = pmpro_url("cancel", "?level=" . $morder->membership_level->id);
                                    }
                                    break;

                                default:

                                    // No manejado, se cancela la suscripcion
                                    $morder->status = "cancelled";
                                    $morder->saveOrder();
                                    $redir_url = pmpro_url("cancel", "?level=" . $morder->membership_level->id);
                                    break;
                            }

                            // Redireccionar a la pantalla correspondiente de pmpro
                            wp_redirect($redir_url);
                            exit();
                        } else {

                            error_log("get_pending_status error (" . $pg->get_rs_code() . " - " . $pg->get_rs_message() . ") token=" . $token . " order=" . $order);
                            $morder->status = "cancelled";
                            $morder->saveOrder();
                            $redir_url = pmpro_url("cancel", "?level=" . $morder->membership_level->id);
                            wp_redirect($redir_url);
                            exit();
                        }
                    } else {

                        error_log("get_oauth_recurring_payments_token error (" . $pg->get_rs_code() . " - " . $pg->get_rs_message() . ") token=" . $token . " order=" . $order);
                        $morder->status = "cancelled";
                        $morder->saveOrder();
                        $redir_url = pmpro_url("cancel", "?level=" . $morder->membership_level->id);
                        wp_redirect($redir_url);
                        exit();
                    }
                } else {

                    // consulta el token del pago unico
                    if ($pg->get_status($token) === true) {

                        switch ($pg->get_rs_status()) {

                            case "VERIFYING":

                                // La transaccion esta pendiente de revision en pagadito
                                $morder->status = "review";
                                $morder->payment_transaction_id = $pg->get_rs_reference();
                                $morder->saveOrder();
                                $redir_url = pmpro_url("confirmation", "?level=" . $morder->membership_level->id);
                                break;

                            case "COMPLETED":

                                // Transaccion completada con exito
                                $morder->status = "success";
                                $morder->payment_transaction_id = $pg->get_rs_reference();
                                $morder->saveOrder();
                                $redir_url = pmpro_url("confirmation", "?level=" . $morder->membership_level->id);
                                if (!self::pagadito_set_membership($morder)) {
                                    $redir_url = pmpro_url("cancel", "?level=" . $morder->membership_level->id);
                                }
                                break;

                            default:

                                // No manejado, se cancela la suscripcion
                                $morder->status = "cancelled";
                                $morder->saveOrder();
                                $redir_url = pmpro_url("cancel", "?level=" . $morder->membership_level->id);
                                break;
                        }

                        // Redireccionar a la pantalla correspondiente de pmpro
                        wp_redirect($redir_url);
                        exit();
                    } else {

                        error_log("get_status error (" . $pg->get_rs_code() . " - " . $pg->get_rs_message() . ") token=" . $token . " order=" . $order);
                        $morder->status = "cancelled";
                        $morder->saveOrder();
                        $redir_url = pmpro_url("cancel", "?level=" . $morder->membership_level->id);
                        wp_redirect($redir_url);
                        exit();
                    }
                }
            } else {

                error_log("Get status no connect (" . $pg->get_rs_code() . " - " . $pg->get_rs_message() . ") token=" . $token . " order=" . $order);
            }
        } catch (Exception $e) {

            error_log("Get status generic token=" . $token . " order=" . $order) . " e=" . $e->getMessage();
        }

        // Se encontro un error, por defecto redireccionar al home de wordpress
        wp_redirect(home_url());
        exit();
    }

    /**
     * Actualiza el nivel de membresia cuando un pago ha sido realizado con exito
     * @param mixed $morder
     * @return boolean
     */
    static function pagadito_set_membership(&$morder) {
        global $wpdb;

        if (!empty($morder->membership_level->expiration_number)) {

            $enddate = "'" . date("Y-m-d", strtotime("+ " . $morder->membership_level->expiration_number . " " . $morder->membership_level->expiration_period, current_time("timestamp"))) . "'";
        } else {

            $enddate = "NULL";
        }
        $morder->getDiscountCode();
        if (!empty($morder->discount_code)) {

            //update membership level
            $morder->getMembershipLevel(true);
            $discount_code_id = $morder->discount_code->id;
        } else {

            $discount_code_id = "";
        }

        $startdate = apply_filters("pmpro_checkout_start_date", "'" . current_time('mysql') . "'", $morder->user_id, $morder->membership_level);

        $custom_level = array(
            'user_id' => $morder->user_id,
            'membership_id' => $morder->membership_level->id,
            'code_id' => $discount_code_id,
            'initial_payment' => $morder->membership_level->initial_payment,
            'billing_amount' => $morder->membership_level->billing_amount,
            'cycle_number' => $morder->membership_level->cycle_number,
            'cycle_period' => $morder->membership_level->cycle_period,
            'billing_limit' => $morder->membership_level->billing_limit,
            'trial_amount' => $morder->membership_level->trial_amount,
            'trial_limit' => $morder->membership_level->trial_limit,
            'startdate' => $startdate,
            'enddate' => $enddate
        );

        if (pmpro_changeMembershipLevel($custom_level, $morder->user_id) !== false) {

            // change order status to succes
            $morder->status = "success";
            $morder->saveOrder();

            // hook
            do_action("pmpro_after_checkout", $morder->user_id);

            //setup some values for the emails
            if (!empty($morder)) {

                $invoice = new MemberOrder($morder->id);
            } else {

                $invoice = NULL;
            }

            $user = get_userdata($morder->user_id);
            $user->membership_level = $morder->membership_level;  //make sure they have the right level info
            //send email to member
            $pmproemail = new PMProEmail();
            $pmproemail->sendCheckoutEmail($user, $invoice);

            //send email to admin
            $pmproemail = new PMProEmail();
            $pmproemail->sendCheckoutAdminEmail($user, $invoice);
            return true;
        } else {

            return false;
        }
    }

    /**
     * No implementado
     * @param mixed $order
     * @return boolean
     */
    function authorize(&$order) {

        return false;
    }

    /**
     * No implementado
     * @param mixed $order
     * @return boolean
     */
    function void(&$order) {

        return false;
    }

    /**
     * No implementado
     * @param mixed $order
     * @return boolean
     */
    function charge(&$order) {

        return false;
    }

    /**
     * No implementado
     * @param mixed $order
     * @return boolean
     */
    function subscribe(&$order) {

        return false;
    }

    /**
     * No implementado
     * @param mixed $order
     * @return boolean
     */
    function update(&$order) {

        return false;
    }

    /**
     * Cancela una suscripcion
     * @param mixed $order
     * @return boolean
     */
    function cancel(&$order) {

        //require a subscription id
        if (empty($order->subscription_transaction_id))
            return false;

        //simulate a successful cancel
        $order->status = "cancelled";
        $order->saveOrder();
        return true;
    }

    function getSubscriptionStatus(&$order) {

        // require a subscription id
        if (empty($order->subscription_transaction_id))
            return false;

        //this looks different for each gateway, but generally an array of some sort
        return array();
    }

    function getTransactionStatus(&$order) {

        return array();
    }

    function pagadito_webhook_listener() {

        // validar firma
        if (self::pagadito_verify_webhook_sign()) {
            
            // obtener la data y pasar a un objeto
            $data = file_get_contents('php://input');
            $obj_data = json_decode($data);

            // validar que el evento sea de cambio de estatus de las transacciones
            if ($obj_data->event_type == 'TRANSACTION.STATUS.CHANGE') {

                // obtener datos de la transaccion
                $webhook_event_id = $obj_data->id;
                $ern = $obj_data->resource->ern;
                $token = $obj_data->resource->token;
                $fecha_creacion = $obj_data->resource->create_timestamp;
                $monto = $obj_data->resource->amount->total;
                $fecha_actualizacion = $obj_data->resource->update_timestamp;
                $status = $obj_data->resource->status;
                $referencia = $obj_data->resource->reference;

                // identificar si el ern es de una recurrencia o de una orden
                $array_ern = explode('-', $ern);
                if (count($array_ern) == 2) {

                    // es un ern de una suscripcion de recurrencia
                    // sacamos el ern del pago original
                    $ern_pago_inicial = $array_ern[0];

                    // sacamos la orden inicial para obtener los datos
                    $orden_pago_inicial = new MemberOrder();
                    if ($orden_pago_inicial->getMemberOrderByCode($ern_pago_inicial) !== false) {

                        // obtenemos el nivel de la membresia
                        $orden_pago_inicial->getMembershipLevel();
                        
                        // obtenemos al usuario de la orden
                        $orden_pago_inicial->getUser();

                        // validamos que la orden sea de tipo recurring
                        if (pmpro_isLevelRecurring($orden_pago_inicial->membership_level)) {

                            // validamos que de la transaccion que viene en el evento webhook
                            if ($status == 'COMPLETED') {

                                // crear una nueva orden con el pago realizado dentro de la mebresia
                                self::pagadito_guardar_orden_pagada($orden_pago_inicial, $webhook_event_id, $ern, $monto, $fecha_actualizacion, $referencia);
                            } else if ($status == 'UNCOLLECTABLE') {

                                // crear una orden cancelada y cancelar la suscripcion
                                self::pagadito_cancelar_suscripcion($orden_pago_inicial, $webhook_event_id, $ern, $token, $fecha_creacion, $monto, $fecha_actualizacion, $referencia);
                            }
                        } else {

                            // el nivel de la orden inicial no es de level recurring
                            error_log("error se recibio un evento webhook de una orden con pago de recurrencia de una orden que no es de level recurring. order_code -> " . $ern);
                        }
                    } else {

                        // la orden inicial no existe
                        error_log("error se recibio un evento webhook de una orden con pago de recurrencia no registrada. order_code -> " . $ern);
                    }
                } else {

                    // es un ern de primer pago
                    // mandamos a cargar la orden
                    $order = new MemberOrder();
                    if ($order->getMemberOrderByCode($ern) !== false) {

                        // ahora depende del estado de la orden actualmente para ver si se va realizar alguna acción
                        if ($order->status == 'review') {
                            
                            // obtenemos el nivel de la membresia de la orden
                            $order->getMembershipLevel();
                            
                            // obtenemos al usuario de la orden
                            $order->getUser();

                            // validamos que de la transaccion que viene en el evento webhook
                            if ($status == 'COMPLETED') {

                                // cambiar estado de la orden y setear la membresia
                                $order->status = "success";
                                $order->notes = "orden completada despues de estar con pago en revisión en fecha [" . $fecha_actualizacion . "] [WEBHOOK_EVENT_ID]" . $webhook_event_id . "[/WEBHOOK_EVENT_ID]";
                                $order->saveOrder();
                                if (!self::pagadito_set_membership($order)) {

                                    error_log("error no se pudo cambiar el nivel de la membresia de un pago en revision que paso a completado. order_code -> " . $ern);
                                }
                            } else if ($status == 'REVOKED') {

                                // cambiar estado de la orden
                                $order->status = "canceled";
                                $order->notes = "orden cancelada despues de estar con pago en revisión que paso a rechazado en fecha [" . $fecha_actualizacion . "] [WEBHOOK_EVENT_ID]" . $webhook_event_id . "[/WEBHOOK_EVENT_ID]";
                                $order->saveOrder();
                            }
                        }
                    } else {

                        // la orden no existe
                        error_log("error se recibio un evento webhook de una orden no registrada. order_code -> " . $ern);
                    }
                }
            }
        }
        exit();
    }

    function pagadito_verify_webhook_sign() {

        // obtener headers
        $headers = getallheaders();
        $notification_id = $headers['PAGADITO-NOTIFICATION-ID'];
        $notification_timestamp = $headers['PAGADITO-NOTIFICATION-TIMESTAMP'];
        $auth_algo = $headers['PAGADITO-AUTH-ALGO'];
        $cert_url = $headers['PAGADITO-CERT-URL'];
        $notification_signature = base64_decode($headers['PAGADITO-SIGNATURE']);

        // obtener data
        $data = file_get_contents('php://input');

        // obtener id evento
        $array_data = json_decode($data, TRUE);
        $event_id = $array_data['id'];

        // setear wsk
        $wsk = pmpro_getOption("pg-wsk");

        // generar cadena para confirmar firma
        $data_signed = $notification_id . '|' . $notification_timestamp . '|' . $event_id . '|' . crc32($data) . '|' . $wsk;

        // obtener contenido del certificado
        // opciones de peticiones http para generar el stream context para obtener el certificado
        $http_options = array(
            'http' => array(
                'protocol_version' => '1.1',
                'method' => 'GET',
                'header' => array(
                    'Connection: close'
                ),
            )
        );
        $cert_stream_context = stream_context_create($http_options);
        $cert_content = file_get_contents($cert_url, FALSE, $cert_stream_context);

        // obtener llave publica
        $pubkeyid = openssl_pkey_get_public($cert_content);

        // verificar firma
        $resultado = openssl_verify($data_signed, $notification_signature, $pubkeyid, $auth_algo);

        // liberar llave publica
        openssl_free_key($pubkeyid);

        // verificacion
        if ($resultado == 1) {

            $sign_verification = TRUE;
        } elseif ($resultado == 0) {

            error_log('verificacion de la firma del webhook invalida');
        } else {

            error_log('error realizando la verificacion de la firma del webhook');
        }
        return $sign_verification;
    }

    function pagadito_guardar_orden_pagada($orden_pago_inicial, $webhook_event_id, $ern, $monto, $fecha_actualizacion, $referencia) {
        global $wpdb;

        // validar que el pago no este procesado previamente
        $old_payment_transaction_id = $wpdb->get_var("SELECT payment_transaction_id FROM $wpdb->pmpro_membership_orders WHERE payment_transaction_id = '" . $referencia . "' LIMIT 1");
        if (empty($old_payment_transaction_id)) {

            // guardar nueva orden
            $order = new MemberOrder();
            $order->code = $ern;
            $order->user_id = $orden_pago_inicial->user_id;
            $order->membership_id = $orden_pago_inicial->membership_id;
            $order->payment_transaction_id = $referencia;
            $order->subscription_transaction_id = $orden_pago_inicial->subscription_transaction_id;
            $order->gateway = $orden_pago_inicial->gateway;
            $order->gateway_environment = $orden_pago_inicial->gateway_environment;
            $order->status = 'success';
            $order->payment_type = $orden_pago_inicial->payment_type;
            // calculo del monto en base al monto de pago de la orden inicial
            // impuestos en el monto total a pagar
            $amount = $orden_pago_inicial->membership_level->billing_amount;
            $amount_tax = $orden_pago_inicial->getTaxForPrice($amount);
            $amount = round((float) $amount + (float) $amount_tax, 2);
            $order->InitialPayment = $amount; //not the initial payment, but the class is expecting that
            $order->PaymentAmount = $amount;
            //figure out timestamp or default to none (today)
            $order->timestamp = strtotime($fecha_actualizacion);
            // Allow extraction of the IPN Track ID from the order notes (if needed)
            $order->notes = "Orden creada desde la recepción de un webhook. [WEBHOOK_EVENT_ID]" . $webhook_event_id . "[/WEBHOOK_EVENT_ID]";

            // guardar orden
            $order->saveOrder();
            $order->getMemberOrderByID($order->id);

            //email the user their invoice
            $pmproemail = new PMProEmail();
            $pmproemail->sendInvoiceEmail(get_userdata($orden_pago_inicial->user_id), $order);

            // hook for successful subscription payments
            do_action("pmpro_subscription_payment_completed", $order);
            return true;
        } else {

            error_log("error pago duplicado. payment_transaction_id -> " . $referencia);
            return false;
        }
    }

    function pagadito_cancelar_suscripcion($orden_pago_inicial, $webhook_event_id, $ern, $monto, $fecha_actualizacion) {

        // buscar si la orden ya esta registrada
        $order = new MemberOrder();
        if ($order->getMemberOrderByCode($ern) == false) {

            // la orden no esta creada
            // crear la orden cancelada
            $order = new MemberOrder();
            $order->code = $ern;
            $order->user_id = $orden_pago_inicial->user_id;
            $order->membership_id = $orden_pago_inicial->membership_id;
            $order->subscription_transaction_id = $orden_pago_inicial->subscription_transaction_id;
            $order->gateway = $orden_pago_inicial->gateway;
            $order->gateway_environment = $orden_pago_inicial->gateway_environment;
            $order->status = 'cancelled';
            $order->payment_type = $orden_pago_inicial->payment_type;
            // calculo del monto en base al monto de pago de la orden inicial
            // impuestos en el monto total a pagar
            $amount = $orden_pago_inicial->membership_level->billing_amount;
            $amount_tax = $orden_pago_inicial->getTaxForPrice($amount);
            $amount = round((float) $amount + (float) $amount_tax, 2);
            $order->InitialPayment = $amount; //not the initial payment, but the class is expecting that
            $order->PaymentAmount = $amount;
            //figure out timestamp or default to none (today)
            $order->timestamp = strtotime($fecha_actualizacion);
            // Allow extraction of the IPN Track ID from the order notes (if needed)
            $order->notes = "Orden cancelada creada desde la recepción de un webhook de transacción incobrable. [WEBHOOK_EVENT_ID]" . $webhook_event_id . "[/WEBHOOK_EVENT_ID]";

            // guardar orden
            $order->saveOrder();
            $order->getMemberOrderByID($order->id);
        }

        // con la orden cargada o recien creada cancelamos la suscripción
        // obtenemos el usuario
        $user = get_userdata($order->user_id);
        
        // verificamos que la orden tenga estado cancelada
        if ($order->status !== 'cancelled') {
            
            error_log("La orden no tiene estado cancelado. Esta membresia probablemente fue cambiada por un administrado o mediante un upgrade/downgrade. order_code -> " . $order->code . ", suscription_transaction_id -> " . $order->suscription_transaction_id);
            return false;
        } else if (isset($order->membership_id) && !pmpro_hasMembershipLevel($order->membership_id, $user->ID)) {
            
            // verificar que el usuario tenga la misma membresia que la asociada a la orden
            error_log("Este usuario tiene un nivel diferente que el asociado a la orden. Esta membresia probablemente fue cambiada por un administrado o mediante un upgrade/downgrade. order_code -> " . $order->code . ", suscription_transaction_id -> " . $order->suscription_transaction_id);
            return false;
        } else {
            
            // realizar el proceso de cancelación de la membresia
            pmpro_changeMembershipLevel(0, $order->user_id, 'cancelled');
            error_log("Membresia cancelada para el usuario con el id -> " . $order->user_id . ", orden_code -> " . $order->code . ", suscription_transaction_id -> " . $order->suscription_transaction_id);

            //send an email to the member
            $pmproemail = new PMProEmail();
            $pmproemail->sendCancelEmail($user, $order->membership_id);

            //send an email to the admin
            $pmproemail = new PMProEmail();
            $pmproemail->sendCancelAdminEmail($user, $order->membership_id);
            return true;
        }
    }

}
?>